@@ -27,6 +27,11 @@
  * CONDITIONS OF OSMC-PL.
  *
  */
+#ifdef USE_PARJAC
+  #include <omp.h>
+  #define GC_THREADS
+  #include <gc/omc_gc.h>
+#endif
 
 #include <string.h>
 #include <setjmp.h>
@@ -36,6 +41,7 @@
 #include "simulation_data.h"
 
 #include "util/omc_error.h"
+#include "util/parallel_helper.h"
 #include "gc/omc_gc.h"
 
 #include "simulation/options.h"
@@ -46,7 +52,7 @@
 #include "simulation/solver/external_input.h"
 #include "simulation/solver/epsilon.h"
 #include "simulation/solver/omc_math.h"
-
+#include "simulation/solver/jacobianSymbolical.h"
 #include "simulation/solver/dassl.h"
 #include "meta/meta_modelica.h"
 
@@ -62,27 +68,49 @@ extern "C" {
 int RHSFinalFlag;
 
 /* provides a dummy Jacobian to be used with DASSL */
-static int
-dummy_Jacobian(double *t, double *y, double *yprime, double *deltaD,
-    double *delta, double *cj, double *h, double *wt, double *rpar, int* ipar) {
-  return 0;
-}
-static int
-dummy_zeroCrossing(int *neqm, double *t, double *y, double *yp,
-                   int *ng, double *gout, double *rpar, int* ipar) {
+static int dummy_Jacobian(double *t, double *y, double *yprime,double *deltaD,
+                          double *delta, double *cj, double *h, double *wt,
+                          double *rpar, int* ipar) {
   return 0;
 }
 
-static int callJacobian(double *t, double *y, double *yprime, double *deltaD, double *pd, double *cj, double *h, double *wt,
-   double *rpar, int* ipar);
-static int jacA_num(double *t, double *y, double *yprime, double *deltaD, double *pd, double *cj, double *h, double *wt,
-   double *rpar, int* ipar);
-static int jacA_numColored(double *t, double *y, double *yprime, double *deltaD, double *pd, double *cj, double *h, double *wt,
-   double *rpar, int* ipar);
-static int jacA_sym(double *t, double *y, double *yprime, double *deltaD, double *pd, double *cj, double *h, double *wt,
-       double *rpar, int* ipar);
-static int jacA_symColored(double *t, double *y, double *yprime, double *deltaD, double *pd, double *cj, double *h, double *wt,
-       double *rpar, int* ipar);
+/* provides a dummy zero crossing function to be used with DASSL */
+static int dummy_zeroCrossing(int *neqm, double *t, double *y, double *yp,
+                              int *ng, double *gout, double *rpar, int* ipar) {
+  return 0;
+}
+
+/* provides a dumm precondition function to be used with DASSL */
+static int dummy_precondition(int *neq, double *t, double *y, double *yprime,
+                              double *savr, double *pwk, double *cj,
+                              double *wt, double *wp, int *iwp, double *b,
+                              double eplin, int* ires, double *rpar, int* ipar){
+    return 0;
+}
+
+/* Function prototypes */
+static int callJacobian(double *t, double *y, double *yprime, double *deltaD,
+                        double *pd, double *cj, double *h, double *wt,
+                        double *rpar, int* ipar);
+
+static int jacA_num(double *t, double *y, double *yprime, double *deltaD,
+                    double *pd, double *cj, double *h, double *wt,
+                    double *rpar, int* ipar);
+
+static int jacA_numColored(double *t, double *y, double *yprime,
+                           double *deltaD, double *pd, double *cj, double *h,
+                           double *wt, double *rpar, int* ipar);
+
+static int jacA_sym(double *t, double *y, double *yprime, double *deltaD,
+                    double *pd, double *cj, double *h, double *wt,
+                    double *rpar, int* ipar);
+
+static int jacA_symColored(double *t, double *y, double *yprime,
+                           double *deltaD, double *pd, double *cj, double *h,
+                           double *wt, double *rpar, int* ipar);
+
+static void setJacElementDasslSparse(int l, int k, int nth, double val,
+                                     void* matrixA, int rows);
 
 void  DDASKR(
     int (*res) (double *t, double *y, double *yprime, double* cj, double *delta, int *ires, double *rpar, int* ipar),
@@ -108,55 +136,51 @@ void  DDASKR(
     int *jroot
 );
 
-static int
-dummy_precondition(int *neq, double *t, double *y, double *yprime, double *savr, double *pwk, double *cj, double *wt, double *wp, int *iwp, double *b, double eplin, int* ires, double *rpar, int* ipar){
-    return 0;
-}
-
 static int continue_DASSL(int* idid, double* tolarence);
 
 /* function for calculating state values on residual form */
-static int functionODE_residual(double *t, double *x, double *xprime, double *cj, double *delta, int *ires, double *rpar, int* ipar);
-/* function for calculating state values on residual form */
-static int functionDAE_residual(double *t, double *x, double *xprime, double *cj, double *delta, int *ires, double *rpar, int* ipar);
-/* function for calculating zeroCrossings */
-static int function_ZeroCrossingsDASSL(int *neqm, double *t, double *y, double *yp,
-        int *ng, double *gout, double *rpar, int* ipar);
+static int functionODE_residual(double *t, double *x, double *xprime,
+                                double *cj, double *delta, int *ires,
+                                double *rpar, int* ipar);
 
-int dassl_initial(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo, DASSL_DATA *dasslData)
+/* function for calculating zeroCrossings */
+static int function_ZeroCrossingsDASSL(int *neqm, double *t, double *y,
+                                       double *yp, int *ng, double *gout,
+                                       double *rpar, int* ipar);
+
+
+/*
+ * \brief Configure DASSL solver
+ *
+ * Allocate memory for intern data of `dasslData`.
+ * Configures DASSL:
+ *   - Set relative and absolute tolerance
+ *   - Set maximum step size, initial step size
+ *   - Set maximum integration order
+ *   - Set time grid
+ *   - Set method for jacobian computation
+ *   - Set root finding method
+ *   - Set event handling and restart option
+ *
+ */
+int dassl_initial(DATA* data, threadData_t *threadData,
+                  SOLVER_INFO* solverInfo, DASSL_DATA *dasslData)
 {
   TRACE_PUSH
   /* work arrays for DASSL */
   unsigned int i;
-  long N, NDAE;
+  long N;
   SIMULATION_DATA tmpSimData = {0};
 
-  dasslData->daeMode = 0;
   dasslData->residualFunction = functionODE_residual;
   N = data->modelData->nStates;
-  NDAE = 0;
 
-  /* change parameter for DAE mode */
-  if (omc_flag[FLAG_DAE_MODE])
-  {
-    if (compiledInDAEMode)
-    {
-      dasslData->daeMode = 1;
-      dasslData->residualFunction = functionDAE_residual;
-      N = data->modelData->nStates + data->simulationInfo->daeModeData->nAlgebraicDAEVars;
-      NDAE = N;
-    }
-    else
-    {
-      warningStreamPrint(LOG_STDOUT, 0, "-daeMode flag is used, the model is not compiled in DAE mode. See compiler flag: +daeMode. Continue as usual.");
-    }
-  }
   dasslData->N = N;
 
   RHSFinalFlag = 0;
 
-  dasslData->liw = 40 + N + NDAE;
-  dasslData->lrw = 60 + ((maxOrder + 4) * N) + (N * N)  + (3*data->modelData->nZeroCrossings)  + NDAE;
+  dasslData->liw = 40 + N;
+  dasslData->lrw = 60 + ((maxOrder + 4) * N) + (N * N)  + (3*data->modelData->nZeroCrossings);
   dasslData->rwork = (double*) calloc(dasslData->lrw, sizeof(double));
   assertStreamPrint(threadData, 0 != dasslData->rwork,"out of memory");
   dasslData->iwork = (int*)  calloc(dasslData->liw, sizeof(int));
@@ -175,11 +199,11 @@ int dassl_initial(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo,
   dasslData->idid = 0;
 
   dasslData->ysave = (double*) malloc(N*sizeof(double));
-  dasslData->ypsave = (double*) malloc(N*sizeof(double));
   dasslData->delta_hh = (double*) malloc(N*sizeof(double));
   dasslData->newdelta = (double*) malloc(N*sizeof(double));
   dasslData->stateDer = (double*) calloc(N, sizeof(double));
   dasslData->states = (double*) malloc(N*sizeof(double));
+  dasslData->allocatedParMem = 0;   /* false */
 
   data->simulationInfo->currentContext = CONTEXT_ALGEBRAIC;
 
@@ -317,16 +341,7 @@ int dassl_initial(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo,
   }
   else
   {
-    /* in dae mode use for now internal Jacobian calculation */
-    if (dasslData->daeMode)
-    {
-      dasslData->dasslJacobian = NUMJAC;
-      warningStreamPrint(LOG_SOLVER, 0, "Running in DAE mode so currently no colored jacobian available!");
-    }
-    else
-    {
-      dasslData->dasslJacobian = COLOREDNUMJAC;
-    }
+    dasslData->dasslJacobian = COLOREDNUMJAC;
   }
 
   /* selects the calculation method of the jacobian */
@@ -334,10 +349,17 @@ int dassl_initial(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo,
      dasslData->dasslJacobian == COLOREDSYMJAC ||
      dasslData->dasslJacobian == SYMJAC)
   {
-    if (data->callback->initialAnalyticJacobianA(data, threadData))
+    ANALYTIC_JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[data->callback->INDEX_JAC_A]);
+    if (data->callback->initialAnalyticJacobianA(data, threadData, jacobian))
     {
       infoStreamPrint(LOG_STDOUT, 0, "Jacobian or SparsePattern is not generated or failed to initialize! Switch back to normal.");
       dasslData->dasslJacobian = INTERNALNUMJAC;
+    } else {
+      ANALYTIC_JACOBIAN* jac = &data->simulationInfo->analyticJacobians[data->callback->INDEX_JAC_A];
+      infoStreamPrint(LOG_SIMULATION, 1, "Initialized colored Jacobian:");
+      infoStreamPrint(LOG_SIMULATION, 0, "columns: %d rows: %d", jac->sizeCols, jac->sizeRows);
+      infoStreamPrint(LOG_SIMULATION, 0, "NNZ:  %d colors: %d", jac->sparsePattern->numberOfNoneZeros, jac->sparsePattern->maxColors);
+      messageClose(LOG_SIMULATION);
     }
   }
   /* default use a user sub-routine for JAC */
@@ -346,15 +368,23 @@ int dassl_initial(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo,
   /* set up the appropriate function pointer */
   switch (dasslData->dasslJacobian){
     case COLOREDNUMJAC:
-      data->simulationInfo->jacobianEvals = data->simulationInfo->analyticJacobians[data->callback->INDEX_JAC_A].sparsePattern.maxColors;
+      data->simulationInfo->jacobianEvals = data->simulationInfo->analyticJacobians[data->callback->INDEX_JAC_A].sparsePattern->maxColors;
       dasslData->jacobianFunction =  jacA_numColored;
       break;
     case COLOREDSYMJAC:
-      data->simulationInfo->jacobianEvals = data->simulationInfo->analyticJacobians[data->callback->INDEX_JAC_A].sparsePattern.maxColors;
+      data->simulationInfo->jacobianEvals = data->simulationInfo->analyticJacobians[data->callback->INDEX_JAC_A].sparsePattern->maxColors;
       dasslData->jacobianFunction =  jacA_symColored;
+#ifdef USE_PARJAC
+      allocateThreadLocalJacobians(data, &(dasslData->jacColumns));
+      dasslData->allocatedParMem = 1;   /* true */
+#endif
       break;
     case SYMJAC:
       dasslData->jacobianFunction =  jacA_sym;
+#ifdef USE_PARJAC
+      allocateThreadLocalJacobians(data, &(dasslData->jacColumns));
+      dasslData->allocatedParMem = 1;   /* true */
+#endif
       break;
     case NUMJAC:
       dasslData->jacobianFunction =  jacA_num;
@@ -397,25 +427,17 @@ int dassl_initial(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo,
   }
   infoStreamPrint(LOG_SOLVER, 0, "dassl performs an restart after an event occurs %s", dasslData->dasslAvoidEventRestart?"NO":"YES");
 
-  /* configure algebraic variables as such */
-  if (dasslData->daeMode)
-  {
-    dasslData->info[15] = 1;
-
-    for(i=0; i<dasslData->N; ++i)
-    {
-      dasslData->iwork[40 + i] =  (i<data->modelData->nStates)? 1.0: -1.0;
-    }
-  }
   /* ### end configuration of dassl ### */
 
-
   messageClose(LOG_SOLVER);
   TRACE_POP
   return 0;
 }
 
 
+/*
+ * \brief Deallocates `DASSL_DATA`
+ */
 int dassl_deinitial(DASSL_DATA *dasslData)
 {
   TRACE_PUSH
@@ -424,17 +446,25 @@ int dassl_deinitial(DASSL_DATA *dasslData)
   /* free work arrays for DASSL */
   free(dasslData->rwork);
   free(dasslData->iwork);
+  free(dasslData->jroot);
   free(dasslData->rpar);
   free(dasslData->ipar);
   free(dasslData->atol);
   free(dasslData->rtol);
   free(dasslData->info);
-  free(dasslData->jroot);
   free(dasslData->ysave);
   free(dasslData->delta_hh);
   free(dasslData->newdelta);
-  free(dasslData->states);
   free(dasslData->stateDer);
+  free(dasslData->states);
+
+
+#ifdef USE_PARJAC
+  if (dasslData->allocatedParMem) {
+      freeAnalyticalJacobian(&(dasslData->jacColumns));
+      dasslData->allocatedParMem = 0;
+  }
+#endif
 
   free(dasslData);
 
@@ -511,16 +541,15 @@ int dassl_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo)
   SIMULATION_DATA *sData = data->localData[0];
   SIMULATION_DATA *sDataOld = data->localData[1];
 
-  modelica_real* states = (dasslData->daeMode)?dasslData->states:sData->realVars;
+  modelica_real* states = sData->realVars;
   modelica_real* stateDer = dasslData->stateDer;
 
 
   MODEL_DATA *mData = (MODEL_DATA*) data->modelData;
 
-  if (!dasslData->daeMode)
-  {
-    memcpy(stateDer, data->localData[1]->realVars + data->modelData->nStates, sizeof(double)*data->modelData->nStates);
-  }
+  if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
+
+  memcpy(stateDer, data->localData[1]->realVars + data->modelData->nStates, sizeof(double)*data->modelData->nStates);
 
   dasslData->rpar[0] = (double*) (void*) data;
   dasslData->rpar[1] = (double*) (void*) dasslData;
@@ -544,12 +573,6 @@ int dassl_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo)
     dasslData->info[0] = 0;
     dasslData->idid = 0;
 
-    if (dasslData->daeMode)
-    {
-      memcpy(states, data->localData[0]->realVars, sizeof(double)*data->modelData->nStates);
-      data->simulationInfo->daeModeData->getAlgebraicDAEVars(data, threadData, states + data->modelData->nStates);
-      memcpy(stateDer, data->localData[1]->realVars + data->modelData->nStates, sizeof(double)*data->modelData->nStates);
-    }
   }
 
   /* Calculate steps until TOUT is reached */
@@ -597,9 +620,11 @@ int dassl_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo)
     /* rhs final flag is FALSE during for dassl evaluation */
     RHSFinalFlag = 0;
 
+    if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
     /* read input vars */
     externalInputUpdate(data);
     data->callback->input_function(data, threadData);
+    if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
 
     DDASKR(dasslData->residualFunction, (int*) &dasslData->N,
             &solverInfo->currentTime, states, stateDer, &tout,
@@ -664,16 +689,7 @@ int dassl_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo)
 
   } while(dasslData->idid == 1);
 
-  if (dasslData->daeMode)
-  {
-    memcpy(data->localData[0]->realVars, states, sizeof(double)*data->modelData->nStates);
-    data->simulationInfo->daeModeData->setAlgebraicDAEVars(data, threadData, states + data->modelData->nStates);
-    memcpy(data->localData[0]->realVars + data->modelData->nStates, stateDer, sizeof(double)*data->modelData->nStates);
-  }
-  else
-  {
-    states = dasslData->states;
-  }
+  states = dasslData->states;
 
 #if !defined(OMC_EMCC)
   MMC_CATCH_INTERNAL(simulationJumpBuffer)
@@ -713,13 +729,13 @@ int dassl_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo)
   }
 
   infoStreamPrint(LOG_DASSL, 0, "Finished DASSL step.");
+  if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
 
   TRACE_POP
   return retVal;
 }
 
-static int
-continue_DASSL(int* idid, double* atol)
+static int continue_DASSL(int* idid, double* atol)
 {
   TRACE_PUSH
   int retValue = -1;
@@ -782,9 +798,8 @@ continue_DASSL(int* idid, double* atol)
   return retValue;
 }
 
-
-int functionODE_residual(double *t, double *y, double *yd, double* cj, double *delta,
-                    int *ires, double *rpar, int *ipar)
+int functionODE_residual(double *t, double *y, double *yd, double* cj,
+                         double *delta, int *ires, double *rpar, int *ipar)
 {
   TRACE_PUSH
   DATA* data = (DATA*)((double**)rpar)[0];
@@ -796,6 +811,9 @@ int functionODE_residual(double *t, double *y, double *yd, double* cj, double *d
   int saveJumpState;
   int success = 0;
 
+  if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
+  if (measure_time_flag) rt_tick(SIM_TIMER_RESIDUALS);
+
   if (data->simulationInfo->currentContext == CONTEXT_ALGEBRAIC)
   {
     setContext(data, t, CONTEXT_ODE);
@@ -845,86 +863,15 @@ int functionODE_residual(double *t, double *y, double *yd, double* cj, double *d
     unsetContext(data);
   }
 
-  TRACE_POP
-  return 0;
-}
-
-int functionDAE_residual(double *t, double *y, double *yd, double* cj, double *delta,
-                    int *ires, double *rpar, int *ipar)
-{
-  TRACE_PUSH
-  DATA* data = (DATA*)((double**)rpar)[0];
-  DASSL_DATA* dasslData = (DASSL_DATA*)((double**)rpar)[1];
-  threadData_t *threadData = (threadData_t*)((double**)rpar)[2];
-
-  double timeBackup;
-  long i;
-  int saveJumpState;
-  int success = 0;
-
-  /* context */
-  if (data->simulationInfo->currentContext == CONTEXT_ALGEBRAIC)
-  {
-    setContext(data, t, CONTEXT_ODE);
-  }
-
-  /* debug */
-  if (ACTIVE_STREAM(LOG_DASSL_STATES)){
-    printCurrentStatesVector(LOG_DASSL_STATES, y, data, *t);
-    printVector(LOG_DASSL_STATES, "yprime", yd, data->modelData->nStates, *t);
-  }
-
-  timeBackup = data->localData[0]->timeValue;
-  data->localData[0]->timeValue = *t;
-
-  memcpy(data->localData[0]->realVars, y, sizeof(double)*data->modelData->nStates);
-  memcpy(data->localData[0]->realVars + data->modelData->nStates, yd, sizeof(double)*data->modelData->nStates);
-  data->simulationInfo->daeModeData->setAlgebraicDAEVars(data, threadData,  y + data->modelData->nStates);
-
-  saveJumpState = threadData->currentErrorStage;
-  threadData->currentErrorStage = ERROR_INTEGRATOR;
-
-  /* try */
-#if !defined(OMC_EMCC)
-  MMC_TRY_INTERNAL(simulationJumpBuffer)
-#endif
-
-  /* read input vars */
-  externalInputUpdate(data);
-  data->callback->input_function(data, threadData);
-
-  /* eval residual vars */
-  data->simulationInfo->daeModeData->evaluateDAEResiduals(data, threadData);
-
-  /* get data->simulationInfo->residualVars  */
-  for(i=0; i < data->simulationInfo->daeModeData->nResidualVars; i++)
-  {
-    delta[i] = data->simulationInfo->daeModeData->residualVars[i];
-  }
-  printVector(LOG_DASSL_STATES, "residual", delta, data->simulationInfo->daeModeData->nResidualVars, *t);
-  success = 1;
-#if !defined(OMC_EMCC)
-  MMC_CATCH_INTERNAL(simulationJumpBuffer)
-#endif
-
-  if (!success) {
-    *ires = -1;
-  }
-
-  threadData->currentErrorStage = saveJumpState;
-
-  data->localData[0]->timeValue = timeBackup;
-
-  if (data->simulationInfo->currentContext == CONTEXT_ODE){
-    unsetContext(data);
-  }
+  if (measure_time_flag) rt_accumulate(SIM_TIMER_RESIDUALS);
+  if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
 
   TRACE_POP
   return 0;
 }
 
 int function_ZeroCrossingsDASSL(int *neqm, double *t, double *y, double *yp,
-        int *ng, double *gout, double *rpar, int* ipar)
+                                int *ng, double *gout, double *rpar, int* ipar)
 {
   TRACE_PUSH
   DATA* data = (DATA*)(void*)((double**)rpar)[0];
@@ -934,6 +881,9 @@ int function_ZeroCrossingsDASSL(int *neqm, double *t, double *y, double *yp,
   double timeBackup;
   int saveJumpState;
 
+  if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
+  if (measure_time_flag) rt_tick(SIM_TIMER_EVENT);
+
   if (data->simulationInfo->currentContext == CONTEXT_ALGEBRAIC)
   {
     setContext(data, t, CONTEXT_EVENTS);
@@ -945,13 +895,6 @@ int function_ZeroCrossingsDASSL(int *neqm, double *t, double *y, double *yp,
   timeBackup = data->localData[0]->timeValue;
   data->localData[0]->timeValue = *t;
 
-  if (dasslData->daeMode)
-  {
-    memcpy(data->localData[0]->realVars, y, sizeof(double)*data->modelData->nStates);
-    memcpy(data->localData[0]->realVars + data->modelData->nStates, yp, sizeof(double)*data->modelData->nStates);
-    data->simulationInfo->daeModeData->setAlgebraicDAEVars(data, threadData,  y + data->modelData->nStates);
-  }
-
   /* read input vars */
   externalInputUpdate(data);
   data->callback->input_function(data, threadData);
@@ -967,53 +910,59 @@ int function_ZeroCrossingsDASSL(int *neqm, double *t, double *y, double *yp,
     unsetContext(data);
   }
 
+  if (measure_time_flag) rt_accumulate(SIM_TIMER_EVENT);
+  if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
+
   TRACE_POP
   return 0;
 }
 
+/*
+ * Sets element (l,j) in matrixA to given value val.
+ */
+void setJacElementDasslSparse(int l, int j, int nth, double val, void* matrixA,
+                              int rows)
+{
+  int k  = j*rows + l;
+  ((double*) matrixA)[k]=val;
+}
+
 /* \fn jacA_symColored(double *t, double *y, double *yprime, double *deltaD, double *pd, double *cj, double *h, double *wt,
    double *rpar, int* ipar)
  *
  *
  * This function calculates symbolically the jacobian matrix and exploiting the coloring.
  */
-int jacA_symColored(double *t, double *y, double *yprime, double *delta, double *matrixA, double *cj, double *h, double *wt, double *rpar, int *ipar)
+int jacA_symColored(double *t, double *y, double *yprime, double *delta,
+                    double *matrixA, double *cj, double *h, double *wt,
+                    double *rpar, int *ipar)
 {
   TRACE_PUSH
   DATA* data = (DATA*)(void*)((double**)rpar)[0];
-  DASSL_DATA* dasslData = (DASSL_DATA*)(void*)((double**)rpar)[1];
   threadData_t *threadData = (threadData_t*)(void*)((double**)rpar)[2];
-
+  DASSL_DATA* dasslData = (DASSL_DATA*)(void*)((double**)rpar)[1];
   const int index = data->callback->INDEX_JAC_A;
-  unsigned int i,j,l,k,ii;
+  ANALYTIC_JACOBIAN* jac = &(data->simulationInfo->analyticJacobians[index]);
 
-  for(i=0; i < data->simulationInfo->analyticJacobians[index].sparsePattern.maxColors; i++)
-  {
-    for(ii=0; ii < data->simulationInfo->analyticJacobians[index].sizeCols; ii++)
-      if(data->simulationInfo->analyticJacobians[index].sparsePattern.colorCols[ii]-1 == i)
-        data->simulationInfo->analyticJacobians[index].seedVars[ii] = 1;
+#ifdef USE_PARJAC
+  ANALYTIC_JACOBIAN* t_jac = (dasslData->jacColumns);
+#else
+  ANALYTIC_JACOBIAN* t_jac = jac;
+#endif
 
-    data->callback->functionJacA_column(data, threadData);
-
-    for(j = 0; j < data->simulationInfo->analyticJacobians[index].sizeCols; j++)
-    {
-      if(data->simulationInfo->analyticJacobians[index].seedVars[j] == 1)
-      {
-        ii = data->simulationInfo->analyticJacobians[index].sparsePattern.leadindex[j];
-        while(ii < data->simulationInfo->analyticJacobians[index].sparsePattern.leadindex[j+1])
-        {
-          l  = data->simulationInfo->analyticJacobians[index].sparsePattern.index[ii];
-          k  = j*data->simulationInfo->analyticJacobians[index].sizeRows + l;
-          matrixA[k] = data->simulationInfo->analyticJacobians[index].resultVars[l];
-          ii++;
-        };
-      }
-    }
-    for(ii=0; ii < data->simulationInfo->analyticJacobians[index].sizeCols; ii++)
-      if(data->simulationInfo->analyticJacobians[index].sparsePattern.colorCols[ii]-1 == i) data->simulationInfo->analyticJacobians[index].seedVars[ii] = 0;
+  unsigned int columns = jac->sizeCols;
+  unsigned int rows = jac->sizeRows;
+  unsigned int sizeTmpVars = jac->sizeTmpVars;
+  SPARSE_PATTERN* spp = jac->sparsePattern;
 
+  /* Evaluate constant equations if available */
+  if (jac->constantEqns != NULL) {
+      jac->constantEqns(data, threadData, jac, NULL);
   }
 
+  genericColoredSymbolicJacobianEvaluation(rows, columns, spp, matrixA, t_jac,
+                                           data, threadData, &setJacElementDasslSparse);
+
   TRACE_POP
   return 0;
 }
@@ -1023,31 +972,65 @@ int jacA_symColored(double *t, double *y, double *yprime, double *delta, double
  *
  *
  * This function calculates symbolically the jacobian matrix.
+ * Can calculate the jacobian in parallel.
  */
-int jacA_sym(double *t, double *y, double *yprime, double *delta, double *matrixA, double *cj, double *h, double *wt, double *rpar, int *ipar)
+int jacA_sym(double *t, double *y, double *yprime, double *delta,
+             double *matrixA, double *cj, double *h, double *wt, double *rpar,
+             int *ipar)
 {
   TRACE_PUSH
+
   DATA* data = (DATA*)(void*)((double**)rpar)[0];
   DASSL_DATA* dasslData = (DASSL_DATA*)(void*)((double**)rpar)[1];
   threadData_t *threadData = (threadData_t*)(void*)((double**)rpar)[2];
+
   const int index = data->callback->INDEX_JAC_A;
-  unsigned int i,j,k;
+  ANALYTIC_JACOBIAN* jac = &(data->simulationInfo->analyticJacobians[index]);
+  unsigned int columns = jac->sizeCols;
+  unsigned int rows = jac->sizeRows;
+  unsigned int sizeTmpVars = jac->sizeTmpVars;
+  unsigned int i;
 
-  k = 0;
-  for(i=0; i < data->simulationInfo->analyticJacobians[index].sizeCols; i++)
-  {
-    data->simulationInfo->analyticJacobians[index].seedVars[i] = 1.0;
-
-    data->callback->functionJacA_column(data, threadData);
-
-    for(j = 0; j < data->simulationInfo->analyticJacobians[index].sizeRows; j++)
-    {
-      matrixA[k++] = data->simulationInfo->analyticJacobians[index].resultVars[j];
-    }
-
-    data->simulationInfo->analyticJacobians[index].seedVars[i] = 0.0;
+  /* Evaluate constant equations if available */
+  if (jac->constantEqns != NULL) {
+      jac->constantEqns(data, threadData, jac, NULL);
   }
 
+#ifdef USE_PARJAC
+  GC_allow_register_threads();
+#endif
+
+#pragma omp parallel default(none) firstprivate(columns, rows, sizeTmpVars) shared(i, matrixA, data, threadData, dasslData)
+{
+#ifdef USE_PARJAC
+  /* Register omp-thread in GC */
+  if(!GC_thread_is_registered()) {
+     struct GC_stack_base sb;
+     memset (&sb, 0, sizeof(sb));
+     GC_get_stack_base(&sb);
+     GC_register_my_thread (&sb);
+  }
+  // Use thread local analytic Jacobians
+  ANALYTIC_JACOBIAN* t_jac = &(dasslData->jacColumns[omc_get_thread_num()]);
+  //printf("index= %d, t_jac->sizeCols= %d, t_jac->sizeRows = %d, t_jac->sizeTmpVars = %d \n",index, t_jac->sizeCols , t_jac->sizeRows, t_jac->sizeTmpVars);
+#else
+  ANALYTIC_JACOBIAN* t_jac = jac;
+#endif
+  unsigned int j;
+
+#pragma omp for schedule(runtime)
+  for(i=0; i < columns; i++)
+  {
+    t_jac->seedVars[i] = 1.0;
+    data->callback->functionJacA_column(data, threadData, t_jac, NULL);
+
+    for(j = 0; j < rows; j++)
+      matrixA[i*columns+j] = t_jac->resultVars[j];
+
+    t_jac->seedVars[i] = 0.0;
+  } // for loop
+} // omp parallel
+
   TRACE_POP
   return 0;
 }
@@ -1059,7 +1042,9 @@ int jacA_sym(double *t, double *y, double *yprime, double *delta, double *matrix
  * This function calculates a jacobian matrix by
  * numerical with forward finite differences.
  */
-int jacA_num(double *t, double *y, double *yprime, double *delta, double *matrixA, double *cj, double *h, double *wt, double *rpar, int *ipar)
+int jacA_num(double *t, double *y, double *yprime, double *delta,
+             double *matrixA, double *cj, double *h, double *wt, double *rpar,
+             int *ipar)
 {
   TRACE_PUSH
   DATA* data = (DATA*)(void*)((double**)rpar)[0];
@@ -1069,10 +1054,11 @@ int jacA_num(double *t, double *y, double *yprime, double *delta, double *matrix
   double delta_h = numericalDifferentiationDeltaXsolver;
   double delta_hh,delta_hhh, deltaInv;
   double ysave;
-  double ypsave;
   int ires;
   int i,j;
 
+  /* set context for the start values extrapolation of non-linear algebraic loops */
+  setContext(data, t, CONTEXT_JACOBIAN);
 
   for(i=dasslData->N-1; i >= 0; i--)
   {
@@ -1083,10 +1069,6 @@ int jacA_num(double *t, double *y, double *yprime, double *delta, double *matrix
     deltaInv = 1. / delta_hh;
     ysave = y[i];
     y[i] += delta_hh;
-    if (dasslData->daeMode){
-      ypsave = yprime[i];
-      yprime[i] += *cj * delta_hh;
-    }
 
     /* internal dassl numerical jacobian is
      * calculated by adding cj to yprime.
@@ -1103,9 +1085,6 @@ int jacA_num(double *t, double *y, double *yprime, double *delta, double *matrix
       matrixA[i*dasslData->N+j] = (dasslData->newdelta[j] - delta[j]) * deltaInv;
     }
     y[i] = ysave;
-    if (dasslData->daeMode){
-      yprime[i] = ypsave;
-    }
   }
 
   TRACE_POP
@@ -1119,7 +1098,9 @@ int jacA_num(double *t, double *y, double *yprime, double *delta, double *matrix
  * This function calculates a jacobian matrix by
  * numerical with forward finite differences and exploiting the coloring.
  */
-int jacA_numColored(double *t, double *y, double *yprime, double *delta, double *matrixA, double *cj, double *h, double *wt, double *rpar, int *ipar)
+int jacA_numColored(double *t, double *y, double *yprime, double *delta,
+                    double *matrixA, double *cj, double *h, double *wt,
+                    double *rpar, int *ipar)
 {
   TRACE_PUSH
 
@@ -1128,20 +1109,24 @@ int jacA_numColored(double *t, double *y, double *yprime, double *delta, double
   threadData_t *threadData = (threadData_t*)(void*)((double**)rpar)[2];
 
   const int index = data->callback->INDEX_JAC_A;
+  ANALYTIC_JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[index]);
+
   double delta_h = numericalDifferentiationDeltaXsolver;
   double delta_hhh;
   int ires;
   double* delta_hh = dasslData->delta_hh;
   double* ysave = dasslData->ysave;
-  double* ypsave = dasslData->ypsave;
 
   unsigned int i,j,l,k,ii;
 
-  for(i = 0; i < data->simulationInfo->analyticJacobians[index].sparsePattern.maxColors; i++)
+  /* set context for the start values extrapolation of non-linear algebraic loops */
+  setContext(data, t, CONTEXT_JACOBIAN);
+
+  for(i = 0; i < jacobian->sparsePattern->maxColors; i++)
   {
-    for(ii=0; ii < data->simulationInfo->analyticJacobians[index].sizeCols; ii++)
+    for(ii=0; ii < jacobian->sizeCols; ii++)
     {
-      if(data->simulationInfo->analyticJacobians[index].sparsePattern.colorCols[ii]-1 == i)
+      if(jacobian->sparsePattern->colorCols[ii]-1 == i)
       {
         delta_hhh = *h * yprime[ii];
         delta_hh[ii] = delta_h * fmax(fmax(fabs(y[ii]),fabs(delta_hhh)),fabs(1./wt[ii]));
@@ -1151,37 +1136,26 @@ int jacA_numColored(double *t, double *y, double *yprime, double *delta, double
         ysave[ii] = y[ii];
         y[ii] += delta_hh[ii];
 
-        if (dasslData->daeMode){
-          ypsave[ii] = yprime[ii];
-          yprime[ii] += *cj * delta_hh[ii];
-        }
-
-
         delta_hh[ii] = 1. / delta_hh[ii];
       }
     }
-
     (*dasslData->residualFunction)(t, y, yprime, cj, dasslData->newdelta, &ires, rpar, ipar);
 
     increaseJacContext(data);
 
-    for(ii = 0; ii < data->simulationInfo->analyticJacobians[index].sizeCols; ii++)
+    for(ii = 0; ii < jacobian->sizeCols; ii++)
     {
-      if(data->simulationInfo->analyticJacobians[index].sparsePattern.colorCols[ii]-1 == i)
+      if(jacobian->sparsePattern->colorCols[ii]-1 == i)
       {
-        j = data->simulationInfo->analyticJacobians[index].sparsePattern.leadindex[ii];
-        while(j < data->simulationInfo->analyticJacobians[index].sparsePattern.leadindex[ii+1])
+        j = jacobian->sparsePattern->leadindex[ii];
+        while(j < jacobian->sparsePattern->leadindex[ii+1])
         {
-          l  =  data->simulationInfo->analyticJacobians[index].sparsePattern.index[j];
-          k  = l + ii*data->simulationInfo->analyticJacobians[index].sizeRows;
+          l  =  jacobian->sparsePattern->index[j];
+          k  = l + ii*jacobian->sizeRows;
           matrixA[k] = (dasslData->newdelta[l] - delta[l]) * delta_hh[ii];
           j++;
         };
         y[ii] = ysave[ii];
-        if (dasslData->daeMode)
-        {
-          yprime[ii] = ypsave[ii];
-        }
       }
     }
   }
@@ -1197,18 +1171,17 @@ int jacA_numColored(double *t, double *y, double *yprime, double *delta, double
  * This function is called by dassl to calculate the jacobian matrix.
  *
  */
-static int callJacobian(double *t, double *y, double *yprime, double *deltaD, double *pd, double *cj, double *h, double *wt,
-   double *rpar, int* ipar)
+static int callJacobian(double *t, double *y, double *yprime, double *deltaD,
+                        double *pd, double *cj, double *h, double *wt,
+                        double *rpar, int* ipar)
 {
   TRACE_PUSH
   DATA* data = (DATA*)(void*)((double**)rpar)[0];
   DASSL_DATA* dasslData = (DASSL_DATA*)(void*)((double**)rpar)[1];
   threadData_t *threadData = (threadData_t*)(void*)((double**)rpar)[2];
 
-  /* set context for the start values extrapolation of non-linear algebraic loops */
-  setContext(data, t, CONTEXT_JACOBIAN);
-
   /* profiling */
+  if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
   rt_tick(SIM_TIMER_JACOBIAN);
 
   if(dasslData->jacobianFunction(t, y, yprime, deltaD, pd, cj, h, wt, rpar, ipar))
@@ -1218,9 +1191,6 @@ static int callJacobian(double *t, double *y, double *yprime, double *deltaD, do
     return 1;
   }
 
-  /* profiling */
-  rt_accumulate(SIM_TIMER_JACOBIAN);
-
   /* debug */
   if (ACTIVE_STREAM(LOG_JAC)){
     _omc_matrix* dumpJac = _omc_createMatrix(dasslData->N, dasslData->N, pd);
@@ -1228,19 +1198,19 @@ static int callJacobian(double *t, double *y, double *yprime, double *deltaD, do
     _omc_destroyMatrix(dumpJac);
   }
 
-  /* add cj to diagonal elements and store in pd */
-  if (!dasslData->daeMode)
+  int i,j = 0;
+  for(i = 0; i < dasslData->N; i++)
   {
-    int i,j = 0;
-    for(i = 0; i < dasslData->N; i++)
-    {
-      pd[j] -= (double) *cj;
-      j += dasslData->N + 1;
-    }
+    pd[j] -= (double) *cj;
+    j += dasslData->N + 1;
   }
   /* set context for the start values extrapolation of non-linear algebraic loops */
   unsetContext(data);
 
+  /* profiling */
+  rt_accumulate(SIM_TIMER_JACOBIAN);
+  if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
+
   TRACE_POP
   return 0;
 }
