@@ -391,10 +391,6 @@ void TextAnnotation::drawTextAnnotaion(QPainter *painter)
   // draw the font
   if (mpComponent || mappedBoundingRect.width() != 0 || mappedBoundingRect.height() != 0) {
     painter->drawText(mappedBoundingRect, StringHandler::getTextAlignment(mHorizontalAlignment) | Qt::AlignVCenter | Qt::TextDontClip, textToDraw);
-    mExportBoundingRect = painter->boundingRect(mappedBoundingRect, StringHandler::getTextAlignment(mHorizontalAlignment) | Qt::AlignVCenter | Qt::TextDontClip, textToDraw);
-    if (mpComponent) {
-      mExportBoundingRect = sceneTransform().mapRect(mExportBoundingRect);
-    }
   }
 }
 
@@ -542,6 +538,7 @@ void TextAnnotation::updateTextStringHelper(QRegExp regExp)
     QString variable = regExp.cap(0).trimmed();
     if ((!variable.isEmpty()) && (variable.compare("%%") != 0) && (variable.compare("%name") != 0) && (variable.compare("%class") != 0)) {
       variable.remove("%");
+      variable = StringHandler::removeFirstLastCurlBrackets(variable);
       if (!variable.isEmpty()) {
         QString textValue;
         /* Ticket:4204
@@ -551,11 +548,11 @@ void TextAnnotation::updateTextStringHelper(QRegExp regExp)
         if (!textValue.isEmpty()) {
           QString unit = "";
           QString displaytUnit = "";
-          Element *pComponent = mpComponent->getRootParentComponent()->getComponentByName(variable);
-          if (pComponent) {
-            displaytUnit = pComponent->getDerivedClassModifierValue("displaytUnit");
+          Element *pElement = mpComponent->getRootParentComponent()->getElementByName(variable);
+          if (pElement) {
+            displaytUnit = pElement->getDerivedClassModifierValue("displaytUnit");
             if (displaytUnit.isEmpty()) {
-              unit = pComponent->getDerivedClassModifierValue("unit");
+              unit = pElement->getDerivedClassModifierValue("unit");
               displaytUnit = unit;
             }
           }
@@ -563,6 +560,7 @@ void TextAnnotation::updateTextStringHelper(QRegExp regExp)
             mTextString.replace(pos, regExp.matchedLength(), textValue);
             pos += textValue.length();
           } else {
+            displaytUnit = Utilities::convertUnitToSymbol(displaytUnit);
             QString textValueWithDisplayUnit = QString("%1 %2").arg(textValue, displaytUnit);
             mTextString.replace(pos, regExp.matchedLength(), textValueWithDisplayUnit);
             pos += textValueWithDisplayUnit.length();
@@ -620,9 +618,9 @@ void TextAnnotation::updateTextString()
       return;
     }
     /* handle variables now */
-    updateTextStringHelper(QRegExp("(%%|%\\w*)"));
+    updateTextStringHelper(QRegExp("(%%|%\\{?\\w+(\\.\\w+)*\\}?)"));
     /* call again with non-word characters so invalid % can be removed. */
-    updateTextStringHelper(QRegExp("(%%|%\\W*)"));
+    updateTextStringHelper(QRegExp("(%%|%\\{?\\W+(\\.\\W+)*\\}?)"));
     /* handle %% */
     if (mOriginalTextString.toLower().contains("%%")) {
       mTextString.replace(QRegExp("%%"), "%");
