@@ -1204,14 +1204,11 @@ void DocumentationViewer::createActions()
 
 /*!
  * \brief DocumentationViewer::resetZoom
- * Resets the zoom. \n
- * QWebView seems to be using fixed 96 dpi so set a proper base zoomfactor for high resolution screens.
+ * Resets the zoom.
  */
 void DocumentationViewer::resetZoom()
 {
-  QWidget *pScreenWidget = QApplication::desktop()->screen();
-  qreal zoomFactor = pScreenWidget->logicalDpiX() / 96;
-  setZoomFactor(zoomFactor < 1 ? 1 : zoomFactor);
+  setZoomFactor(1.0);
 }
 
 /*!
@@ -1374,12 +1371,20 @@ void DocumentationViewer::keyPressEvent(QKeyEvent *event)
  */
 void DocumentationViewer::wheelEvent(QWheelEvent *event)
 {
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 6, 0))
+  if (event->angleDelta().y() != 0 && event->modifiers().testFlag(Qt::ControlModifier)) {
+#else // QT_VERSION_CHECK
   if (event->orientation() == Qt::Vertical && event->modifiers().testFlag(Qt::ControlModifier)) {
+#endif // QT_VERSION_CHECK
     qreal zf = zoomFactor();
     /* ticket:4349 Take smaller steps for zooming.
      * Also set the minimum zoom to readable size.
      */
-    if (event->delta() > 0) {
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 6, 0))
+  if (event->angleDelta().y() > 0) {
+#else // QT_VERSION_CHECK
+  if (event->delta() > 0) {
+#endif // QT_VERSION_CHECK
       zf += 0.1;
       zf = zf > 5 ? 5 : zf;
     } else {
