@@ -1,7 +1,7 @@
 #!/bin/bash
 set -e
 
-testcases=( "BrowseMSL" "Diagram" "Transformation")
+testcases=( "BrowseMSL" "Diagram" "Transformation" "Homotopy" )
 OMEditTestResults="$PWD/OMEditTestResult"
 
 for i in "${testcases[@]}"
@@ -16,7 +16,7 @@ do
   export TEMP=$NEW_TEMP
   export TMP=$NEW_TMP
   export TMPDIR=$NEW_TMPDIR
-  ./$i
+  ./$i || ./$i || ./$i || ./$i || ./$i
   export TEMP=$ORIGINAL_TEMP
   export TMP=$ORIGINAL_TMP
   export TMPDIR=$ORIGINAL_TMPDIR
